require 'io/console'
require 'netrc'

module Octokom
  class Client
    private_class_method :new

    attr_reader :token

    KEY = 'api.github.com'
    URL = 'https://github.com/mjio/octokom'

    def self.authenticate
      new.authenticate
    end

    # Use the existing credentials form the ~/.netrc file to verify API
    # requests to GitHub. When no credentials exist, ask for login and
    # password to request a new token from GitHub.
    def authenticate
      _, @token = netrc[KEY]

      unless token
        login, password = ask_for_credentials
        create_authorization(login, password)
      end

      self
    end

    def client
      require 'octokit'
      Octokit::Client.new(access_token: token)
    end

    private

    def path
      @path ||= Netrc.default_path
    end

    def file
      @file ||= File.open(path, File::RDWR|File::CREAT, 0600)
    end

    def data
      @data ||= Netrc.parse(Netrc.lex(file.each_line.to_a))
    end

    def netrc
      @netrc ||= Netrc.new(path, data)
    end

    def ask_for_credentials
      login = ask_for_login
      password = ask_for_password
      puts # newline
      [login, password]
    end

    def ask_for_login
      print 'GitHub Login: '
      STDIN.gets.chomp
    end

    def ask_for_password
      print 'Password: '
      STDIN.noecho(&:gets).chomp
    end

    def create_authorization(login, password)
      require 'octokit'

      begin
        client = Octokit::Client.new(login: login, password: password)
        auth   = client.create_authorization(scopes: ['repo'], note: api_note, note_url: URL)
      rescue Octokit::Unauthorized
        puts "Unable to authorise user @#{login}"
        exit(1)
      end

      @token = auth.token
      save_token_to_netrc(login)
    end

    def save_token_to_netrc(login)
      netrc[KEY] = login, @token
      netrc.save
    end

    def api_note
      "Octokom #{hostname}@#{Time.now.usec}"
    end

    def hostname
      `hostname`.chomp
    end
  end
end
