class PullRequest < Octokom::Command
  sub_command 'create', 'Create a pull request.' do
    option ['--title',  '-t'], 'Title for the pull request.'
    option ['--desc',   '-d'], 'Description for the pull request.'
    option ['--base',   '-b'], 'The branch you want your changes pulled into. (e.g. master)'
    option ['--head',   '-h'], 'The branch where your changes are implemented. (e.g. feature)'
    option ['--repo',   '-r'], 'The GitHub repository you want to update. (e.g. mjio/octokom)'
    flag   ['--editor', '-e'], 'Open the editor even when the pull request title is set.'

    execute do
      @base ||= git.base_branch
      @head ||= git.head_branch
      @repo ||= git.repo_path

      if @base == @head
        error("Base and head branch can not be the same (#{@base}:#{@head})")
      end

      if @editor || @title.blank?
        @title, @desc = Octokom::Editor.open(git, @title, @desc)
      end

      if @title.blank?
        error('The title can not be empty')
      end

      if git.pending_commits?
        task('Push latests commits') do
          git.push(@head) unless dry_run?
        end
      end

      authenticate do |auth|
        task("Create pull-request #{@repo} (#{@base}:#{@head})") do
          unless dry_run?
            @success = auth.client.create_pull_request(@repo, @base, @head, @title, @desc)
            print " (+#{@success.additions} -#{@success.deletions})"
          end
        end
      end

      puts "#{Octokit::Default::WEB_ENDPOINT}/#{@repo}/pull/#{@success.number}" if @success
    end
  end
end
