module Octokom
  class Repository
    include Octokom::Shell

    def initialize(verbose)
      @verbose = verbose
    end

    def repo_path
      error('Git remote not defined') if remote_url.blank?
      path = remote_url.split('/')[-2..-1]
      path.join('/')[/(.*)\.git$/, 1]
    end

    def toplevel_path
      git('rev-parse --show-toplevel')
    end

    # TODO Make base branch configurable.
    def base_branch
      'master'
    end

    def head_branch
      git('rev-parse --abbrev-ref HEAD')
    end

    def pending_commits?
      current_sha1 = last_commits[/^\*\s((\w|-|_)*)\s+(\w{7})/, 3]
      remote_sha1 = last_commits[/remotes\/origin\/#{head_branch}\s+(\w{7})/, 1]

      return true if remote_sha1.blank?
      current_sha1 != remote_sha1 ? true : false
    end

    def last_commit_message
      git('log -1 --oneline')[/^\w+\s(.*)$/, 1]
    end

    def push(branch)
      git("push origin #{branch}")
    end

    private

    # TODO Make origin configurable.
    def remote_url
      @remote_url ||= git('config --get remote.origin.url')
    end

    def last_commits
      @last_commits ||= git('branch -avvv')
    end

    def verbose?
      !!@verbose
    end

    def git(cmd)
      puts "git #{cmd}" if verbose?
      `git #{cmd}`.chomp
    end
  end
end
